@@ -33,7 +33,7 @@
 #include "item_windowfunc.h"
 
 bool mysql_union(THD *thd, LEX *lex, select_result *result,
-                 SELECT_LEX_UNIT *unit, ulong setup_tables_done_option)
+                 SELECT_LEX_UNIT *unit, ulonglong setup_tables_done_option)
 {
   DBUG_ENTER("mysql_union");
   bool res;
@@ -355,8 +355,6 @@ select_unit::create_result_table(THD *thd_arg, List<Item> *column_types,
     return TRUE;
 
   table->keys_in_use_for_query.clear_all();
-  for (uint i=0; i < table->s->fields; i++)
-    table->field[i]->flags &= ~(PART_KEY_FLAG | PART_INDIRECT_KEY_FLAG);
 
   if (create_table)
   {
@@ -395,9 +393,6 @@ select_union_recursive::create_result_table(THD *thd_arg,
     return true;
 
   incr_table->keys_in_use_for_query.clear_all();
-  for (uint i=0; i < table->s->fields; i++)
-    incr_table->field[i]->flags &= ~(PART_KEY_FLAG | PART_INDIRECT_KEY_FLAG);
-
   return false;
 }
 
@@ -485,7 +480,7 @@ bool select_unit_ext::disable_index_if_needed(SELECT_LEX *curr_sl)
         !curr_sl->next_select()) )
   {
     is_index_enabled= false;
-    if (table->file->ha_disable_indexes(key_map(0), false))
+    if (table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL))
       return false;
     table->no_keyread=1;
     return true;
@@ -547,7 +542,7 @@ int select_unit::delete_record()
   tables of JOIN - exec_tmp_table_[1 | 2].
 */
 
-void select_unit::reset_for_next_ps_execution()
+void select_unit::cleanup()
 {
   table->file->extra(HA_EXTRA_RESET_STATE);
   table->file->ha_delete_all_rows();
@@ -902,11 +897,11 @@ bool select_unit_ext::send_eof()
   return (MY_TEST(error));
 }
 
-void select_union_recursive::reset_for_next_ps_execution()
+void select_union_recursive::cleanup()
 {
   if (table)
   {
-    select_unit::reset_for_next_ps_execution();
+    select_unit::cleanup();
     free_tmp_table(thd, table);
   }
 
@@ -1026,8 +1021,7 @@ bool select_union_direct::send_eof()
   // Reset for each SELECT_LEX, so accumulate here
   limit_found_rows+= thd->limit_found_rows;
 
-  if (unit->thd->lex->current_select == last_select_lex ||
-      thd->killed == ABORT_QUERY)
+  if (unit->thd->lex->current_select == last_select_lex)
   {
     thd->limit_found_rows= limit_found_rows;
 
@@ -1300,6 +1294,7 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
   bool instantiate_tmp_table= false;
   bool single_tvc= !first_sl->next_select() && first_sl->tvc;
   bool single_tvc_wo_order= single_tvc && !first_sl->order_list.elements;
+  bool distinct_key= 0;
   DBUG_ENTER("st_select_lex_unit::prepare");
   DBUG_ASSERT(thd == current_thd);
 
@@ -1403,15 +1398,17 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
     {
     case INTERSECT_TYPE:
       have_intersect= TRUE;
-      if (!s->distinct){
-        have_except_all_or_intersect_all= true;
-      }
+      if (!s->distinct)
+        have_except_all_or_intersect_all= TRUE;
       break;
     case EXCEPT_TYPE:
       have_except= TRUE;
-      if (!s->distinct){
+      if (!s->distinct)
         have_except_all_or_intersect_all= TRUE;
-      }
+      break;
+    case DERIVED_TABLE_TYPE:
+      if (s->distinct)
+        distinct_key= 1;
       break;
     default:
       break;
@@ -1618,7 +1615,8 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
         if (join_union_item_types(thd, types, union_part_count + 1))
           goto err;
         if (union_result->create_result_table(thd, &types,
-                                              MY_TEST(union_distinct),
+                                              (MY_TEST(union_distinct) ||
+                                               distinct_key),
                                               create_options,
                                               &derived_arg->alias, false,
                                               instantiate_tmp_table, false,
@@ -1641,7 +1639,7 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
 
           res= derived_arg->derived_result->create_result_table(thd,
                                                             &types,
-                                                            FALSE,
+                                                            distinct_key,
                                                             create_options,
                                                             &derived_arg->alias,
                                                             FALSE, FALSE,
@@ -1769,9 +1767,9 @@ cont:
         union_result->create_result_table(thd, &types,
                                           MY_TEST(union_distinct) ||
                                             have_except_all_or_intersect_all ||
-                                            have_intersect,
-                                          create_options, &empty_clex_str, false,
-                                          instantiate_tmp_table, false,
+                                            have_intersect || distinct_key,
+                                          create_options, &empty_clex_str,
+                                          false, instantiate_tmp_table, false,
                                           hidden);
       union_result->addon_cnt= hidden;
       for (uint i= 0; i < hidden; i++)   
@@ -2085,7 +2083,7 @@ bool st_select_lex_unit::optimize()
       /* re-enabling indexes for next subselect iteration */
       if ((union_result->force_enable_index_if_needed() || union_distinct))
       {
-        if(table->file->ha_enable_indexes(key_map(table->s->keys), false))
+        if(table->file->ha_enable_indexes(HA_KEY_SWITCH_ALL))
           DBUG_ASSERT(0);
         else
           table->no_keyread= 0;
@@ -2158,8 +2156,8 @@ bool st_select_lex_unit::exec()
   ulonglong add_rows=0;
   ha_rows examined_rows= 0;
   bool first_execution= !executed;
-  DBUG_ENTER("st_select_lex_unit::exec");
   bool was_executed= executed;
+  DBUG_ENTER("st_select_lex_unit::exec");
 
   if (executed && !uncacheable && !describe)
     DBUG_RETURN(FALSE);
@@ -2185,14 +2183,14 @@ bool st_select_lex_unit::exec()
         union_result->table && union_result->table->is_created())
     {
       union_result->table->file->ha_delete_all_rows();
-      union_result->table->file->ha_enable_indexes(key_map(table->s->keys), false);
+      union_result->table->file->ha_enable_indexes(HA_KEY_SWITCH_ALL);
     }
   }
 
   if (uncacheable || !item || !item->assigned() || describe)
   {
     if (!fake_select_lex && !(with_element && with_element->is_recursive))
-      union_result->reset_for_next_ps_execution();
+      union_result->cleanup();
     for (SELECT_LEX *sl= select_cursor; sl; sl= sl->next_select())
     {
       ha_rows records_at_start= 0;
@@ -2246,18 +2244,16 @@ bool st_select_lex_unit::exec()
 	if (sl->tvc)
 	  sl->tvc->exec(sl);
 	else
-	  sl->join->exec();
+	  saved_error= sl->join->exec();
         if (sl == union_distinct && !have_except_all_or_intersect_all &&
             !(with_element && with_element->is_recursive))
 	{
           // This is UNION DISTINCT, so there should be a fake_select_lex
           DBUG_ASSERT(fake_select_lex != NULL);
-	  if (table->file->ha_disable_indexes(key_map(0), false))
+	  if (unlikely(table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL)))
 	    DBUG_RETURN(TRUE);
 	  table->no_keyread=1;
 	}
-	if (!sl->tvc)
-	  saved_error= sl->join->error;
 	if (likely(!saved_error))
 	{
 	  examined_rows+= thd->get_examined_row_count();
@@ -2404,7 +2400,8 @@ bool st_select_lex_unit::exec()
         {
           join->join_examined_rows= 0;
           saved_error= join->reinit();
-          join->exec();
+          if (join->exec())
+            saved_error= 1;
         }
       }
 
@@ -2510,8 +2507,7 @@ bool st_select_lex_unit::exec_recursive()
       sl->tvc->exec(sl);
     else
     {
-      sl->join->exec();
-      saved_error= sl->join->error;
+      saved_error= sl->join->exec();
     }
     if (likely(!saved_error))
     {
@@ -2523,11 +2519,10 @@ bool st_select_lex_unit::exec_recursive()
 	 DBUG_RETURN(1);
        }
     }
-    if (unlikely(saved_error))
+    else
     {
       thd->lex->current_select= lex_select_save;
       goto err;
-      
     }
   }
 
@@ -2634,7 +2629,7 @@ bool st_select_lex_unit::cleanup()
   {
     if (union_result)
     {
-      ((select_union_recursive *) union_result)->reset_for_next_ps_execution();
+      ((select_union_recursive *) union_result)->cleanup();
       delete union_result;
       union_result= 0;
     }
