@@ -29,6 +29,7 @@
 #pragma implementation				// gcc: Class implementation
 #endif
 
+#define VER "1.1"
 #include "mariadb.h"
 #if !defined(TZINFO2SQL) && !defined(TESTTIME)
 #include "sql_priv.h"
@@ -43,6 +44,7 @@
 #include <my_getopt.h>
 #endif
 
+#include <welcome_copyright_notice.h>
 #include "tztime.h"
 #include "tzfile.h"
 #include <m_string.h>
@@ -64,8 +66,6 @@
 #endif /* !defined(DBUG_OFF) */
 #endif /* defined(TZINFO2SQL) || defined(TESTTIME) */
 
-#define PROGRAM_VERSION "1.1"
-
 /* Structure describing local time type (e.g. Moscow summer time (MSD)) */
 typedef struct ttinfo
 {
@@ -1028,9 +1028,9 @@ class Time_zone_system : public Time_zone
 {
 public:
   Time_zone_system() = default;                       /* Remove gcc warning */
-  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const override;
-  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
-  const String * get_name() const override;
+  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const;
+  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
+  virtual const String * get_name() const;
 };
 
 
@@ -1124,10 +1124,10 @@ class Time_zone_utc : public Time_zone
 {
 public:
   Time_zone_utc() = default;                          /* Remove gcc warning */
-  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const override;
-  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
-  const String * get_name() const override;
+  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                                    uint *error_code) const;
+  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
+  virtual const String * get_name() const;
 };
 
 
@@ -1207,9 +1207,9 @@ class Time_zone_db : public Time_zone
 {
 public:
   Time_zone_db(TIME_ZONE_INFO *tz_info_arg, const String * tz_name_arg);
-  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const override;
-  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
-  const String * get_name() const override;
+  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const;
+  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
+  virtual const String * get_name() const;
 private:
   TIME_ZONE_INFO *tz_info;
   const String *tz_name;
@@ -1305,10 +1305,10 @@ class Time_zone_offset : public Time_zone
 {
 public:
   Time_zone_offset(long tz_offset_arg);
-  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const override;
-  void   gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
-  const String * get_name() const override;
+  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                                    uint *error_code) const;
+  virtual void   gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
+  virtual const String * get_name() const;
   /*
     This have to be public because we want to be able to access it from
     my_offset_tzs_get_key() function
@@ -1498,20 +1498,21 @@ public:
   they should obey C calling conventions.
 */
 
-static const uchar *my_tz_names_get_key(const void *entry_, size_t *length,
-                                            my_bool)
+extern "C" uchar *
+my_tz_names_get_key(Tz_names_entry *entry, size_t *length,
+                    my_bool not_used __attribute__((unused)))
 {
-  auto entry= static_cast<const Tz_names_entry *>(entry_);
   *length= entry->name.length();
-  return reinterpret_cast<const uchar *>(entry->name.ptr());
+  return (uchar*) entry->name.ptr();
 }
 
-static const uchar *my_offset_tzs_get_key(const void *entry_,
-                                             size_t *length, my_bool)
+extern "C" uchar *
+my_offset_tzs_get_key(Time_zone_offset *entry,
+                      size_t *length,
+                      my_bool not_used __attribute__((unused)))
 {
-  auto entry= static_cast<const Time_zone_offset *>(entry_);
   *length= sizeof(long);
-  return reinterpret_cast<const uchar *>(&entry->offset);
+  return (uchar*) &entry->offset;
 }
 
 
@@ -1620,18 +1621,20 @@ my_tz_init(THD *org_thd, const char *default_tzname, my_bool bootstrap)
   */
   if (!(thd= new THD(0)))
     DBUG_RETURN(1);
-  thd->thread_stack= (void*) &thd;              // Big stack
+  thd->thread_stack= (char*) &thd;
   thd->store_globals();
+  thd->set_query_inner((char*) STRING_WITH_LEN("intern:my_tz_init"),
+                       default_charset_info);
 
   /* Init all memory structures that require explicit destruction */
   if (my_hash_init(key_memory_tz_storage, &tz_names, &my_charset_latin1, 20, 0,
-                   0, my_tz_names_get_key, 0, 0))
+                   0, (my_hash_get_key) my_tz_names_get_key, 0, 0))
   {
     sql_print_error("Fatal error: OOM while initializing time zones");
     goto end;
   }
   if (my_hash_init(key_memory_tz_storage, &offset_tzs, &my_charset_latin1, 26,
-                   0, 0, my_offset_tzs_get_key, 0, 0))
+                   0, 0, (my_hash_get_key)my_offset_tzs_get_key, 0, 0))
   {
     sql_print_error("Fatal error: OOM while initializing time zones");
     my_hash_free(&tz_names);
@@ -2003,7 +2006,11 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
 #endif
 
     /* ttid is increasing because we are reading using index */
-    DBUG_ASSERT(ttid >= tmp_tz_info.typecnt);
+    if (ttid < tmp_tz_info.typecnt)
+    {
+      sql_print_error("mysql.time_zone_transition_type table is incorrectly defined or corrupted");
+      goto end;
+    }
 
     tmp_tz_info.typecnt= ttid + 1;
 
@@ -2287,8 +2294,6 @@ str_to_offset(const char *str, uint length, long *offset)
     specification or other error.
 
 */
-PRAGMA_DISABLE_CHECK_STACK_FRAME
-
 Time_zone *
 my_tz_find(THD *thd, const String *name)
 {
@@ -2357,7 +2362,6 @@ my_tz_find(THD *thd, const String *name)
 
   DBUG_RETURN(result_tz);
 }
-PRAGMA_REENABLE_CHECK_STACK_FRAME
 
 
 /**
@@ -2519,7 +2523,7 @@ scan_tz_dir(char * name_end, uint symlink_recursion_level, uint verbose)
 {
   MY_DIR *cur_dir;
   char *name_end_tmp;
-  uint i;
+  size_t i;
 
   /* Sort directory data, to pass mtr tests on different platforms. */
   if (!(cur_dir= my_dir(fullname, MYF(MY_WANT_STAT|MY_WANT_SORT))))
@@ -2664,12 +2668,6 @@ static my_bool get_one_option(const struct my_option *, const char *,
                               const char *);
 C_MODE_END
 
-static void print_version(void)
-{
-  printf("%s  Ver %s Distrib %s, for %s (%s)\n",my_progname, PROGRAM_VERSION,
-	 MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE);
-}
-
 static const char *default_timezone_dir= "/usr/share/zoneinfo/";
 
 
@@ -2769,7 +2767,7 @@ main(int argc, char **argv)
 
   printf("set @wsrep_is_on=(%s);\n", wsrep_is_on);
   printf("SET STATEMENT SQL_MODE='' FOR "
-         "SELECT concat('%%', GROUP_CONCAT(OPTION ORDER BY OPTION DESC), '%%') INTO @replicate_opt "
+         "SELECT concat('%%', GROUP_CONCAT(OPTION), '%%') INTO @replicate_opt "
          " FROM"
          "   (SELECT DISTINCT concat('REPLICATE_', UPPER(ENGINE)) AS OPTION"
          "    FROM information_schema.TABLES"
@@ -2780,7 +2778,8 @@ main(int argc, char **argv)
          "                         'time_zone_transition_type',"
          "                         'time_zone_leap_second')"
          "      AND ENGINE in ('MyISAM',"
-         "                     'Aria')) AS o;\n");
+         "                     'Aria')) AS o"
+         " ORDER BY OPTION DESC;\n");
   printf("set @wsrep_cannot_replicate_tz=@wsrep_is_on AND (%s);\n", wsrep_cannot_replicate_tz);
   if (opt_skip_write_binlog)
     /* We turn off session wsrep if we cannot replicate using galera.
