@@ -33,7 +33,6 @@ Created 11/29/1995 Heikki Tuuri
 #include "page0page.h"
 #include "srv0srv.h"
 #include "srv0start.h"
-#include "ibuf0ibuf.h"
 #include "btr0btr.h"
 #include "btr0sea.h"
 #include "dict0boot.h"
@@ -42,8 +41,6 @@ Created 11/29/1995 Heikki Tuuri
 #include "fsp0types.h"
 #include "log.h"
 
-typedef uint32_t page_no_t;
-
 /** Returns the first extent descriptor for a segment.
 We think of the extent lists of the segment catenated in the order
 FSEG_FULL -> FSEG_NOT_FULL -> FSEG_FREE.
@@ -335,7 +332,7 @@ xdes_t*
 xdes_get_descriptor_with_space_hdr(
 	buf_block_t*		header,
 	const fil_space_t*	space,
-	page_no_t		offset,
+	uint32_t		offset,
 	mtr_t*			mtr,
 	dberr_t*		err = nullptr,
 	buf_block_t**		desc_block = nullptr,
@@ -352,7 +349,7 @@ xdes_get_descriptor_with_space_hdr(
 	ut_ad(limit == space->free_limit
 	      || (space->free_limit == 0
 		  && (init_space
-		      || space->is_temporary()
+		      || space->purpose == FIL_TYPE_TEMPORARY
 		      || (srv_startup_is_before_trx_rollback_phase
 			  && (space->id == TRX_SYS_SPACE
 			      || srv_is_undo_tablespace(space->id))))));
@@ -399,7 +396,7 @@ try to add new extents to the space free list
 @param[out]	err		error code
 @param[out]	xdes		extent descriptor page
 @return the extent descriptor */
-static xdes_t *xdes_get_descriptor(const fil_space_t *space, page_no_t offset,
+static xdes_t *xdes_get_descriptor(const fil_space_t *space, uint32_t offset,
                                    mtr_t *mtr, dberr_t *err= nullptr,
                                    buf_block_t **xdes= nullptr)
 {
@@ -446,9 +443,8 @@ Returns page offset of the first page in extent described by a descriptor.
 static uint32_t xdes_get_offset(const xdes_t *descr)
 {
   ut_ad(descr);
-  const page_t *page= page_align(descr);
-  return page_get_page_no(page) +
-    uint32_t(((descr - page - XDES_ARR_OFFSET) / XDES_SIZE) *
+  return page_get_page_no(page_align(descr)) +
+    uint32_t(((page_offset(descr) - XDES_ARR_OFFSET) / XDES_SIZE) *
              FSP_EXTENT_SIZE);
 }
 
@@ -460,8 +456,7 @@ void fsp_apply_init_file_page(buf_block_t *block)
   const page_id_t id(block->page.id());
 
   mach_write_to_4(block->page.frame + FIL_PAGE_OFFSET, id.page_no());
-  if (log_sys.is_physical())
-    memset_aligned<8>(block->page.frame + FIL_PAGE_PREV, 0xff, 8);
+  memset_aligned<8>(block->page.frame + FIL_PAGE_PREV, 0xff, 8);
   mach_write_to_4(block->page.frame + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID,
                   id.space());
   if (page_zip_des_t* page_zip= buf_block_get_page_zip(block))
@@ -471,8 +466,7 @@ void fsp_apply_init_file_page(buf_block_t *block)
     static_assert(FIL_PAGE_OFFSET == 4, "compatibility");
     memcpy_aligned<4>(page_zip->data + FIL_PAGE_OFFSET,
                       block->page.frame + FIL_PAGE_OFFSET, 4);
-    if (log_sys.is_physical())
-      memset_aligned<8>(page_zip->data + FIL_PAGE_PREV, 0xff, 8);
+    memset_aligned<8>(page_zip->data + FIL_PAGE_PREV, 0xff, 8);
     static_assert(FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID % 4 == 2,
                   "not perfect alignment");
     memcpy_aligned<2>(page_zip->data + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID,
@@ -490,15 +484,14 @@ void fil_space_t::modify_check(const mtr_t& mtr) const
   case MTR_LOG_NONE:
     /* These modes are only allowed within a non-bitmap page
        when there is a higher-level redo log record written. */
-    ut_ad(!is_being_imported());
+    ut_ad(purpose == FIL_TYPE_TABLESPACE || purpose == FIL_TYPE_TEMPORARY);
     break;
   case MTR_LOG_NO_REDO:
-    ut_ad(is_temporary() || is_being_imported());
+    ut_ad(purpose == FIL_TYPE_TEMPORARY || purpose == FIL_TYPE_IMPORT);
     break;
   default:
     /* We may only write redo log for a persistent tablespace. */
-    ut_ad(!is_temporary());
-    ut_ad(!is_being_imported());
+    ut_ad(purpose == FIL_TYPE_TABLESPACE);
     ut_ad(mtr.is_named_space(id));
   }
 }
@@ -514,7 +507,7 @@ dberr_t fsp_header_init(fil_space_t *space, uint32_t size, mtr_t *mtr)
 	const page_id_t page_id(space->id, 0);
 	const ulint zip_size = space->zip_size();
 
-	buf_block_t *free_block = buf_LRU_get_free_block(false);
+	buf_block_t *free_block = buf_LRU_get_free_block(have_no_mutex);
 
 	mtr->x_lock_space(space);
 
@@ -848,9 +841,8 @@ fsp_fill_free_list(
 
       if (i)
       {
-        buf_block_t *f= buf_LRU_get_free_block(false);
-        buf_block_t *block= buf_page_create(space, static_cast<uint32_t>(i),
-                               zip_size, mtr, f);
+        buf_block_t *f= buf_LRU_get_free_block(have_no_mutex);
+        buf_block_t *block= buf_page_create(space, i, zip_size, mtr, f);
         if (UNIV_UNLIKELY(block != f))
           buf_pool.free_block(f);
         fsp_init_file_page(space, block, mtr);
@@ -858,15 +850,20 @@ fsp_fill_free_list(
                       FIL_PAGE_TYPE_XDES);
       }
 
-      if (!space->is_temporary())
+      if (space->purpose != FIL_TYPE_TEMPORARY)
       {
-        buf_block_t *f= buf_LRU_get_free_block(false);
+        buf_block_t *f= buf_LRU_get_free_block(have_no_mutex);
         buf_block_t *block=
-          buf_page_create(space,
-                          static_cast<uint32_t>(i + FSP_IBUF_BITMAP_OFFSET),
-                          zip_size, mtr, f);
+          buf_page_create(space, i + 1, zip_size, mtr, f);
         if (UNIV_UNLIKELY(block != f))
           buf_pool.free_block(f);
+        /* The zero-initialization will reset the change buffer bitmap bits
+        to safe values for possible import to an earlier version that
+        supports change buffering:
+
+        IBUF_BITMAP_FREE     = 0 (no space left for buffering inserts)
+        IBUF_BITMAP_BUFFERED = 0 (no changes have been buffered)
+        IBUF_BITMAP_IBUF     = 0 (not part of the change buffer) */
         fsp_init_file_page(space, block, mtr);
         mtr->write<2>(*block, FIL_PAGE_TYPE + block->page.frame,
                       FIL_PAGE_IBUF_BITMAP);
@@ -891,9 +888,9 @@ fsp_fill_free_list(
     if (UNIV_UNLIKELY(init_xdes))
     {
       /* The first page in the extent is a descriptor page and the
-      second is an ibuf bitmap page: mark them used */
+      second was reserved for change buffer bitmap: mark them used */
       xdes_set_free<false>(*xdes, descr, 0, mtr);
-      xdes_set_free<false>(*xdes, descr, FSP_IBUF_BITMAP_OFFSET, mtr);
+      xdes_set_free<false>(*xdes, descr, 1, mtr);
       xdes_set_state(*xdes, descr, XDES_FREE_FRAG, mtr);
       if (dberr_t err= flst_add_last(header, FSP_HEADER_OFFSET + FSP_FREE_FRAG,
                                      xdes, xoffset, space->free_limit, mtr))
@@ -969,7 +966,6 @@ corrupted:
 			first = flst_get_first(FSP_HEADER_OFFSET + FSP_FREE
 					       + header->page.frame);
 			if (first.page == FIL_NULL) {
-				*err = DB_OUT_OF_FILE_SPACE;
 				return nullptr;	/* No free extents left */
 			}
 			if (first.page >= space->free_limit) {
@@ -1053,13 +1049,11 @@ fsp_alloc_from_free_frag(buf_block_t *header, buf_block_t *xdes, xdes_t *descr,
 @param[in]	offset		page number of the allocated page
 @param[in,out]	mtr		mini-transaction
 @return block, initialized */
-static
-buf_block_t*
-fsp_page_create(fil_space_t *space, page_no_t offset, mtr_t *mtr)
+static buf_block_t* fsp_page_create(fil_space_t *space, uint32_t offset,
+                                    mtr_t *mtr)
 {
-  buf_block_t *free_block= buf_LRU_get_free_block(false);
-  buf_block_t *block= buf_page_create(space, static_cast<uint32_t>(offset),
-                                      space->zip_size(), mtr, free_block);
+  buf_block_t *free_block= buf_LRU_get_free_block(have_no_mutex),
+    *block= buf_page_create(space, offset, space->zip_size(), mtr, free_block);
   if (UNIV_UNLIKELY(block != free_block))
     buf_pool.free_block(free_block);
   fsp_init_file_page(space, block, mtr);
@@ -1192,7 +1186,7 @@ MY_ATTRIBUTE((nonnull, warn_unused_result))
 @param[in]      offset  page number in the extent
 @param[in,out]  mtr     mini-transaction
 @return error code */
-static dberr_t fsp_free_extent(fil_space_t* space, page_no_t offset,
+static dberr_t fsp_free_extent(fil_space_t* space, uint32_t offset,
                                mtr_t* mtr)
 {
   ut_ad(space->is_owner());
@@ -1230,7 +1224,7 @@ The page is marked as free and clean.
 @param[in]	offset		page number
 @param[in,out]	mtr		mini-transaction
 @return error code */
-static dberr_t fsp_free_page(fil_space_t *space, page_no_t offset, mtr_t *mtr)
+static dberr_t fsp_free_page(fil_space_t *space, uint32_t offset, mtr_t *mtr)
 {
 	xdes_t*		descr;
 	ulint		frag_n_used;
@@ -1509,8 +1503,7 @@ static void fsp_free_seg_inode(fil_space_t *space, fseg_inode_t *inode,
       return;
   }
 
-  mtr->memset(iblock, inode - iblock->page.frame + FSEG_ID,
-              FSEG_INODE_SIZE, 0);
+  mtr->memset(iblock, page_offset(inode) + FSEG_ID, FSEG_INODE_SIZE, 0);
 
   if (ULINT_UNDEFINED != fsp_seg_inode_page_find_used(iblock->page.frame,
                                                       physical_size))
@@ -1536,7 +1529,7 @@ static
 fseg_inode_t*
 fseg_inode_try_get(
 	const fseg_header_t*	header,
-	ulint			space,
+	uint32_t		space,
 	ulint			zip_size,
 	mtr_t*			mtr,
 	buf_block_t**		block,
@@ -1683,7 +1676,8 @@ fseg_create(fil_space_t *space, ulint byte_offset, mtr_t *mtr, dberr_t *err,
 {
 	fseg_inode_t*	inode;
 	ib_id_t		seg_id;
-	uint32_t	n_reserved = 0;
+	uint32_t	n_reserved;
+	bool		reserved_extent = false;
 
 	DBUG_ENTER("fseg_create");
 
@@ -1711,16 +1705,16 @@ inode_alloc:
 	if (!inode) {
 		block = nullptr;
 reserve_extent:
-		if (!has_done_reservation && !n_reserved) {
+		if (!has_done_reservation && !reserved_extent) {
 			*err = fsp_reserve_free_extents(&n_reserved, space, 2,
 							FSP_NORMAL, mtr);
 			if (UNIV_UNLIKELY(*err != DB_SUCCESS)) {
 				DBUG_RETURN(nullptr);
 			}
 
-			ut_ad(n_reserved > 0);
 			/* Extents reserved successfully. So
 			try allocating the page or inode */
+			reserved_extent = true;
 			if (inode) {
 				goto page_alloc;
 			}
@@ -1780,8 +1774,7 @@ page_alloc:
 	}
 
 	mtr->write<2>(*block, byte_offset + FSEG_HDR_OFFSET
-		      + block->page.frame,
-		      uintptr_t(inode - iblock->page.frame));
+		      + block->page.frame, page_offset(inode));
 
 	mtr->write<4>(*block, byte_offset + FSEG_HDR_PAGE_NO
 		      + block->page.frame, iblock->page.id().page_no());
@@ -1790,8 +1783,7 @@ page_alloc:
 				       + block->page.frame, space->id);
 
 funct_exit:
-	if (n_reserved) {
-		ut_ad(!has_done_reservation);
+	if (!has_done_reservation && reserved_extent) {
 		space->release_free_extents(n_reserved);
 	}
 
@@ -1924,12 +1916,11 @@ fseg_alloc_free_extent(
 	mtr_t*			mtr,
 	dberr_t*		err)
 {
-  ut_ad(iblock->page.frame == page_align(inode));
-  ut_ad(!((inode - iblock->page.frame - FSEG_ARR_OFFSET) % FSEG_INODE_SIZE));
+  ut_ad(!((page_offset(inode) - FSEG_ARR_OFFSET) % FSEG_INODE_SIZE));
   ut_ad(!memcmp(FSEG_MAGIC_N_BYTES, FSEG_MAGIC_N + inode, 4));
   ut_d(space->modify_check(*mtr));
 
-  if (UNIV_UNLIKELY(uintptr_t(inode - iblock->page.frame) < FSEG_ARR_OFFSET))
+  if (UNIV_UNLIKELY(page_offset(inode) < FSEG_ARR_OFFSET))
   {
   corrupted:
     *err= DB_CORRUPTION;
@@ -2173,6 +2164,8 @@ take_hinted_page:
 		buf_block_t* block = fsp_alloc_free_page(
 			space, hint, mtr, init_mtr, err);
 
+		ut_ad(block || !has_done_reservation || *err);
+
 		if (block) {
 			/* Put the page in the fragment page array of the
 			segment */
@@ -2185,8 +2178,6 @@ take_hinted_page:
 			fseg_set_nth_frag_page_no(
 				seg_inode, iblock, n,
 				block->page.id().page_no(), mtr);
-		} else {
-			ut_ad(*err != DB_SUCCESS);
 		}
 
 		/* fsp_alloc_free_page() invoked fsp_init_file_page()
@@ -2286,13 +2277,12 @@ fseg_alloc_free_page_general(
 	dberr_t*	err)	/*!< out: error code */
 {
 	fseg_inode_t*	inode;
-	ulint		space_id;
 	fil_space_t*	space;
 	buf_block_t*	iblock;
 	buf_block_t*	block;
 	uint32_t	n_reserved;
 
-	space_id = page_get_space_id(page_align(seg_header));
+	const uint32_t space_id = page_get_space_id(page_align(seg_header));
 	space = mtr->x_lock_space(space_id);
 	inode = fseg_inode_try_get(seg_header, space_id, space->zip_size(),
 				   mtr, &iblock, err);
@@ -2528,7 +2518,7 @@ fseg_free_page_low(
 	fseg_inode_t*		seg_inode,
 	buf_block_t*		iblock,
 	fil_space_t*		space,
-	page_no_t		offset,
+	uint32_t		offset,
 	mtr_t*			mtr
 #ifdef BTR_CUR_HASH_ADAPT
 	,bool			ahi=false
@@ -2818,18 +2808,33 @@ remove:
 	return DB_SUCCESS;
 }
 
-bool fseg_free_step(buf_block_t *block, size_t header, mtr_t *mtr
+/** Frees part of a segment. This function can be used to free
+a segment by repeatedly calling this function in different
+mini-transactions. Doing the freeing in a single mini-transaction
+might result in too big a mini-transaction.
+@param	header	segment header; NOTE: if the header resides on first
+		page of the frag list of the segment, this pointer
+		becomes obsolete after the last freeing step
+@param	mtr	mini-transaction
+@param	ahi	Drop the adaptive hash index
+@return whether the freeing was completed */
+bool
+fseg_free_step(
+	fseg_header_t*	header,
+	mtr_t*		mtr
 #ifdef BTR_CUR_HASH_ADAPT
-                    , bool ahi
+	,bool		ahi
 #endif /* BTR_CUR_HASH_ADAPT */
-                    ) noexcept
+	)
 {
 	ulint		n;
 	fseg_inode_t*	inode;
 
-	const page_id_t header_id{block->page.id()};
-	fil_space_t* space = mtr->x_lock_space(header_id.space());
-	xdes_t* descr = xdes_get_descriptor(space, header_id.page_no(), mtr);
+	const uint32_t space_id = page_get_space_id(page_align(header));
+	const uint32_t header_page = page_get_page_no(page_align(header));
+
+	fil_space_t* space = mtr->x_lock_space(space_id);
+	xdes_t* descr = xdes_get_descriptor(space, header_page, mtr);
 
 	if (!descr) {
 		return true;
@@ -2839,17 +2844,14 @@ bool fseg_free_step(buf_block_t *block, size_t header, mtr_t *mtr
 	freed yet */
 
 	if (UNIV_UNLIKELY(xdes_is_free(descr,
-				       header_id.page_no()
-				       & (FSP_EXTENT_SIZE - 1)))) {
+				       header_page & (FSP_EXTENT_SIZE - 1)))) {
 		/* Some corruption was detected: stop the freeing
 		in order to prevent a crash. */
 		return true;
 	}
 	buf_block_t* iblock;
 	const ulint zip_size = space->zip_size();
-	inode = fseg_inode_try_get(block->page.frame + header,
-				   header_id.space(), zip_size,
-				   mtr, &iblock);
+	inode = fseg_inode_try_get(header, space_id, zip_size, mtr, &iblock);
 	if (!inode || space->is_stopping()) {
 		return true;
 	}
@@ -2884,7 +2886,7 @@ bool fseg_free_step(buf_block_t *block, size_t header, mtr_t *mtr
 		return true;
 	}
 
-	page_no_t page_no = fseg_get_nth_frag_page_no(inode, n);
+	uint32_t page_no = fseg_get_nth_frag_page_no(inode, n);
 
 	if (fseg_free_page_low(inode, iblock, space, page_no, mtr
 #ifdef BTR_CUR_HASH_ADAPT
@@ -2908,31 +2910,33 @@ bool fseg_free_step(buf_block_t *block, size_t header, mtr_t *mtr
 	return false;
 }
 
-bool fseg_free_step_not_header(buf_block_t *block, size_t header, mtr_t *mtr
+bool
+fseg_free_step_not_header(
+	fseg_header_t*	header,
+	mtr_t*		mtr
 #ifdef BTR_CUR_HASH_ADAPT
-                               , bool ahi
+	,bool		ahi
 #endif /* BTR_CUR_HASH_ADAPT */
-                               ) noexcept
+	)
 {
-	fseg_inode_t* inode;
-	const page_id_t header_id{block->page.id()};
-	ut_ad(mtr->is_named_space(header_id.space()));
+	fseg_inode_t*	inode;
 
-	fil_space_t* space = mtr->x_lock_space(header_id.space());
-	buf_block_t* iblock;
+	const uint32_t space_id = page_get_space_id(page_align(header));
+	ut_ad(mtr->is_named_space(space_id));
 
-	inode = fseg_inode_try_get(block->page.frame + header,
-				   header_id.space(), space->zip_size(),
+	fil_space_t*		space = mtr->x_lock_space(space_id);
+	buf_block_t*		iblock;
+
+	inode = fseg_inode_try_get(header, space_id, space->zip_size(),
 				   mtr, &iblock);
 	if (space->is_stopping()) {
 		return true;
 	}
 
-	if (UNIV_UNLIKELY(!inode)) {
-		sql_print_warning("InnoDB: Double free of page " UINT32PF
-				  " in file %s",
-				  header_id.page_no(),
-				  space->chain.start->name);
+	if (!inode) {
+		ib::warn() << "Double free of "
+			   << page_id_t(space_id,
+					page_get_page_no(page_align(header)));
 		return true;
 	}
 
@@ -2964,7 +2968,7 @@ bool fseg_free_step_not_header(buf_block_t *block, size_t header, mtr_t *mtr
 
 	uint32_t page_no = fseg_get_nth_frag_page_no(inode, n);
 
-	if (page_no == header_id.page_no()) {
+	if (page_no == page_get_page_no(page_align(header))) {
 		return true;
 	}
 
@@ -3043,9 +3047,8 @@ static void fseg_print_low(const fseg_inode_t *inode)
 	ulint	page_no;
 	ib_id_t	seg_id;
 
-	const page_t* inode_page = page_align(inode);
-	space = page_get_space_id(inode_page);
-	page_no = page_get_page_no(inode_page);
+	space = page_get_space_id(page_align(inode));
+	page_no = page_get_page_no(page_align(inode));
 
 	reserved = fseg_n_reserved_pages_low(inode, &used);
 
